@@ -33,6 +33,8 @@ def convert_to_inference_data(obj, *, group="posterior", coords=None, dims=None,
             | emcee sampler: Automatically extracts data
             | pyro MCMC: Automatically extracts data
             | xarray.Dataset: adds to InferenceData as only group
+            | xarray.DataArray: creates an xarray dataset as the only group, gives the
+                         array an arbitrary name, if name not set
             | dict: creates an xarray dataset as the only group
             | numpy array: creates an xarray dataset as the only group, gives the
                          array an arbitrary name
@@ -97,6 +99,10 @@ def convert_to_inference_data(obj, *, group="posterior", coords=None, dims=None,
     # Cases that convert to xarray
     if isinstance(obj, xr.Dataset):
         dataset = obj
+    elif isinstance(obj, xr.DataArray):
+        if obj.name is None:
+            obj.name = "x"
+        dataset = obj.to_dataset()
     elif isinstance(obj, dict):
         dataset = dict_to_dataset(obj, coords=coords, dims=dims)
     elif isinstance(obj, np.ndarray):
@@ -109,6 +115,7 @@ def convert_to_inference_data(obj, *, group="posterior", coords=None, dims=None,
         return from_cmdstan(**kwargs)
     else:
         allowable_types = (
+            "xarray dataarray",
             "xarray dataset",
             "dict",
             "netcdf filename",
@@ -149,6 +156,8 @@ def convert_to_dataset(obj, *, group="posterior", coords=None, dims=None):
             pystan fit: Automatically extracts data
             pymc3 trace: Automatically extracts data
             xarray.Dataset: adds to InferenceData as only group
+            xarray.DataArray: creates an xarray dataset as the only group, gives the
+                         array an arbitrary name, if name not set
             dict: creates an xarray dataset as the only group
             numpy array: creates an xarray dataset as the only group, gives the
                          array an arbitrary name
