@@ -9,22 +9,35 @@ import netCDF4 as nc
 import numpy as np
 import xarray as xr
 
+from ..utils import _subset_list
 from ..rcparams import rcParams
 
 SUPPORTED_GROUPS = [
     "posterior",
-    "sample_stats",
-    "log_likelihood",
     "posterior_predictive",
+    "predictions",
+    "log_likelihood",
+    "sample_stats",
+    "prior",
+    "prior_predictive",
+    "sample_stats_prior",
     "observed_data",
     "constant_data",
-    "prior",
-    "sample_stats_prior",
-    "prior_predictive",
-    "predictions",
     "predictions_constant_data",
 ]
 
+WARMUP_TAG = "warmup_"
+
+SUPPORTED_GROUPS_WARMUP = [
+    "{}posterior".format(WARMUP_TAG),
+    "{}posterior_predictive".format(WARMUP_TAG),
+    "{}predictions".format(WARMUP_TAG),
+    "{}sample_stats".format(WARMUP_TAG),
+    "{}log_likelihood".format(WARMUP_TAG),
+]
+
+SUPPORTED_GROUPS_ALL = SUPPORTED_GROUPS + SUPPORTED_GROUPS_WARMUP
+
 
 class InferenceData:
     """Container for inference data storage using xarray.
@@ -76,36 +89,62 @@ class InferenceData:
 
         """
         self._groups = []
-        key_list = [key for key in SUPPORTED_GROUPS if key in kwargs]
+        self._groups_warmup = []
+        save_warmup = kwargs.pop("save_warmup", False)
+        key_list = [key for key in SUPPORTED_GROUPS_ALL if key in kwargs]
         for key in kwargs:
-            if key not in SUPPORTED_GROUPS:
+            if key not in SUPPORTED_GROUPS_ALL:
                 key_list.append(key)
                 warnings.warn(
                     "{} group is not defined in the InferenceData scheme".format(key), UserWarning
                 )
         for key in key_list:
             dataset = kwargs[key]
+            dataset_warmup = None
             if dataset is None:
                 continue
+            elif isinstance(dataset, (list, tuple)):
+                dataset, dataset_warmup = kwargs[key]
             elif not isinstance(dataset, xr.Dataset):
                 raise ValueError(
                     "Arguments to InferenceData must be xarray Datasets "
                     "(argument '{}' was type '{}')".format(key, type(dataset))
                 )
-            setattr(self, key, dataset)
-            self._groups.append(key)
+            if not key.startswith(WARMUP_TAG):
+                if dataset:
+                    setattr(self, key, dataset)
+                    self._groups.append(key)
+            elif key.startswith(WARMUP_TAG):
+                if dataset:
+                    setattr(self, key, dataset)
+                    self._groups_warmup.append(key)
+            if save_warmup and dataset_warmup is not None:
+                if dataset_warmup:
+                    key = "{}{}".format(WARMUP_TAG, key)
+                    setattr(self, key, dataset_warmup)
+                    self._groups_warmup.append(key)
 
     def __repr__(self):
         """Make string representation of object."""
-        return "Inference data with groups:\n\t> {options}".format(
+        msg = "Inference data with groups:\n\t> {options}".format(
             options="\n\t> ".join(self._groups)
         )
+        if self._groups_warmup:
+            msg += "\n\nWarmup iterations saved ({}*).".format(WARMUP_TAG)
+        return msg
 
     def __delattr__(self, group):
         """Delete a group from the InferenceData object."""
-        self._groups.remove(group)
+        if group in self._groups:
+            self._groups.remove(group)
+        elif group in self._groups_warmup:
+            self._groups_warmup.remove(group)
         object.__delattr__(self, group)
 
+    @property
+    def _groups_all(self):
+        return self._groups + self._groups_warmup
+
     @staticmethod
     def from_netcdf(filename):
         """Initialize object from a netcdf file.
@@ -155,11 +194,12 @@ class InferenceData:
             Location of netcdf file
         """
         mode = "w"  # overwrite first, then append
-        if self._groups:  # check's whether a group is present or not.
+        if self._groups_all:  # check's whether a group is present or not.
             if groups is None:
-                groups = self._groups
+                groups = self._groups_all
             else:
-                groups = [group for group in self._groups if group in groups]
+                groups = [group for group in self._groups_all if group in groups]
+
             for group in groups:
                 data = getattr(self, group)
                 kwargs = {}
@@ -177,26 +217,31 @@ class InferenceData:
         """Concatenate two InferenceData objects."""
         return concat(self, other, copy=True, inplace=False)
 
-    def sel(self, inplace=False, chain_prior=False, **kwargs):
+    def sel(
+        self, groups=None, filter_groups=None, inplace=False, chain_prior=None, **kwargs,
+    ):
         """Perform an xarray selection on all groups.
 
-        Loops over all groups to perform Dataset.sel(key=item)
+        Loops groups to perform Dataset.sel(key=item)
         for every kwarg if key is a dimension of the dataset.
         One example could be performing a burn in cut on the InferenceData object
         or discarding a chain. The selection is performed on all relevant groups (like
         posterior, prior, sample stats) while non relevant groups like observed data are
-        omitted.
+        omitted. See :meth:`xarray.Dataset.sel <xarray:xarray.Dataset.sel>`
 
         Parameters
         ----------
-        inplace : bool, optional
+        groups: str or list of str, optional
+            Groups where the selection is to be applied. Can either be group names
+            or metagroup names.
+        inplace: bool, optional
             If ``True``, modify the InferenceData object inplace,
             otherwise, return the modified copy.
-        chain_prior: bool, optional
+        chain_prior: bool, optional, deprecated
             If ``False``, do not select prior related groups using ``chain`` dim.
-            Otherwise, use selection on ``chain`` if present
-        **kwargs : mapping
-            It must be accepted by Dataset.sel()
+            Otherwise, use selection on ``chain`` if present. Default=False
+        **kwargs: mapping
+            It must be accepted by Dataset.sel().
 
         Returns
         -------
@@ -228,8 +273,18 @@ class InferenceData:
                ...: print(idata_subset.observed_data.coords)
 
         """
+        if chain_prior is not None:
+            warnings.warn(
+                "chain_prior has been deprecated. Use groups argument and "
+                "rcParams['data.metagroups'] instead.",
+                DeprecationWarning,
+            )
+        else:
+            chain_prior = False
+        groups = self._group_names(groups, filter_groups)
+
         out = self if inplace else deepcopy(self)
-        for group in self._groups:
+        for group in groups:
             dataset = getattr(self, group)
             valid_keys = set(kwargs.keys()).intersection(dataset.dims)
             if not chain_prior and "prior" in group:
@@ -241,6 +296,169 @@ class InferenceData:
         else:
             return out
 
+    def _group_names(self, groups, filter_groups=None):
+        """Handle expansion of group names input across arviz.
+
+        Parameters
+        ----------
+        groups: str, list of str or None
+            group or metagroup names.
+        idata: xarray.Dataset
+            Posterior data in an xarray
+        filter_groups: {None, "like", "regex"}, optional, default=None
+            If `None` (default), interpret groups as the real group or metagroup names.
+            If "like", interpret groups as substrings of the real group or metagroup names.
+            If "regex", interpret groups as regular expressions on the real group or
+            metagroup names. A la `pandas.filter`.
+
+        Returns
+        -------
+        groups: list
+        """
+        all_groups = self._groups_all
+        if groups is None:
+            return all_groups
+        if isinstance(groups, str):
+            groups = [groups]
+        sel_groups = []
+        metagroups = rcParams["data.metagroups"]
+        for group in groups:
+            if group[0] == "~":
+                sel_groups.extend(
+                    [f"~{item}" for item in metagroups[group[1:]] if item in all_groups]
+                    if group[1:] in metagroups
+                    else [group]
+                )
+            else:
+                sel_groups.extend(
+                    [item for item in metagroups[group] if item in all_groups]
+                    if group in metagroups
+                    else [group]
+                )
+
+        try:
+            group_names = _subset_list(sel_groups, all_groups, filter_items=filter_groups)
+        except KeyError as err:
+            msg = " ".join(("groups:", f"{err}", "in InferenceData"))
+            raise KeyError(msg)
+        return group_names
+
+    def map(self, fun, groups=None, filter_groups=None, inplace=False, args=None, **kwargs):
+        """Apply a function to multiple groups.
+
+        Parameters
+        ----------
+        fun: callable
+            Function to be applied to each group.
+        groups: str or list of str, optional
+            Groups where the selection is to be applied. Can either be group names
+            or metagroup names.
+        inplace: bool, optional
+            If ``True``, modify the InferenceData object inplace,
+            otherwise, return the modified copy.
+        args: array_like, optional
+            Positional arguments passed to ``fun``. Assumes the function is called as
+            ``fun(dataset, *args, **kwargs)``.
+        **kwargs: mapping, optional
+            Keyword arguments passed to ``fun``.
+
+        Returns
+        -------
+        InferenceData
+            A new InferenceData object by default.
+            When `inplace==True` perform selection in place and return `None`
+
+        Examples
+        --------
+        Shift observed_data, prior_predictive and posterior_predictive.
+
+        .. ipython::
+
+            In [1]: import arviz as az
+               ...: idata = az.load_arviz_data("non_centered_eight")
+               ...: idata_shifted_obs = idata.map(lambda x: x + 3, groups="observed_RVs")
+               ...: print(idata_shifted_obs.observed_data)
+               ...: print(idata_shifted_obs.posterior_predictive)
+
+        """
+        if args is None:
+            args = []
+        groups = self._group_names(groups, filter_groups)
+
+        out = self if inplace else deepcopy(self)
+        for group in groups:
+            dataset = getattr(self, group)
+            dataset = fun(dataset, *args, **kwargs)
+            setattr(out, group, dataset)
+        if inplace:
+            return None
+        else:
+            return out
+
+    def _wrap_xarray_method(
+        self, method, groups=None, filter_groups=None, inplace=False, args=None, **kwargs
+    ):
+        """Extend and xarray.Dataset method to InferenceData object.
+
+        Parameters
+        ----------
+        method: str
+            Method to be extended. Must be a ``xarray.Dataset`` method.
+        groups: str or list of str, optional
+            Groups where the selection is to be applied. Can either be group names
+            or metagroup names.
+        inplace: bool, optional
+            If ``True``, modify the InferenceData object inplace,
+            otherwise, return the modified copy.
+        **kwargs: mapping, optional
+            Keyword arguments passed to the xarray Dataset method.
+
+        Returns
+        -------
+        InferenceData
+            A new InferenceData object by default.
+            When `inplace==True` perform selection in place and return `None`
+
+        Examples
+        --------
+        Compute the mean of `posterior_groups`:
+
+        .. ipython::
+
+            In [1]: import arviz as az
+               ...: idata = az.load_arviz_data("non_centered_eight")
+               ...: idata_means = idata._wrap_xarray_method("mean", groups="latent_RVs")
+               ...: print(idata_means.posterior)
+               ...: print(idata_means.observed_data)
+
+        .. ipython::
+
+            In [1]: idata_stack = idata._wrap_xarray_method(
+               ...:     "stack",
+               ...:     groups=["posterior_groups", "prior_groups"],
+               ...:     sample=["chain", "draw"]
+               ...: )
+               ...: print(idata_stack.posterior)
+               ...: print(idata_stack.prior)
+               ...: print(idata_stack.observed_data)
+
+        """
+        if args is None:
+            args = []
+        groups = self._group_names(groups, filter_groups)
+
+        method = getattr(xr.Dataset, method)
+
+        out = self if inplace else deepcopy(self)
+        for group in groups:
+            dataset = getattr(self, group)
+            dataset = method(dataset, *args, **kwargs)
+            setattr(out, group, dataset)
+        if inplace:
+            return None
+        else:
+            return out
+
 
 # pylint: disable=protected-access, inconsistent-return-statements
 def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
@@ -359,12 +577,12 @@ def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
 
     if dim is None:
         arg0 = args[0]
-        arg0_groups = ccopy(arg0._groups)
+        arg0_groups = ccopy(arg0._groups_all)
         args_groups = dict()
         # check if groups are independent
         # Concat over unique groups
         for arg in args[1:]:
-            for group in arg._groups:
+            for group in arg._groups_all:
                 if group in args_groups or group in arg0_groups:
                     msg = (
                         "Concatenating overlapping groups is not supported unless `dim` is defined."
@@ -374,41 +592,53 @@ def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
                 group_data = getattr(arg, group)
                 args_groups[group] = deepcopy(group_data) if copy else group_data
         # add arg0 to args_groups if inplace is False
+        # otherwise it will merge args_groups to arg0
+        # inference data object
         if not inplace:
             for group in arg0_groups:
                 group_data = getattr(arg0, group)
                 args_groups[group] = deepcopy(group_data) if copy else group_data
 
-        other_groups = [group for group in args_groups if group not in SUPPORTED_GROUPS]
+        other_groups = [group for group in args_groups if group not in SUPPORTED_GROUPS_ALL]
 
-        for group in SUPPORTED_GROUPS + other_groups:
+        for group in SUPPORTED_GROUPS_ALL + other_groups:
             if group not in args_groups:
                 continue
             if inplace:
-                arg0._groups.append(group)
+                if group.startswith(WARMUP_TAG):
+                    arg0._groups_warmup.append(group)
+                else:
+                    arg0._groups.append(group)
                 setattr(arg0, group, args_groups[group])
             else:
                 inference_data_dict[group] = args_groups[group]
         if inplace:
             other_groups = [
-                group for group in arg0_groups if group not in SUPPORTED_GROUPS
+                group for group in arg0_groups if group not in SUPPORTED_GROUPS_ALL
             ] + other_groups
             sorted_groups = [
                 group for group in SUPPORTED_GROUPS + other_groups if group in arg0._groups
             ]
             setattr(arg0, "_groups", sorted_groups)
+            sorted_groups_warmup = [
+                group
+                for group in SUPPORTED_GROUPS_WARMUP + other_groups
+                if group in arg0._groups_warmup
+            ]
+            setattr(arg0, "_groups_warmup", sorted_groups_warmup)
     else:
         arg0 = args[0]
-        arg0_groups = arg0._groups
+        arg0_groups = arg0._groups_all
         for arg in args[1:]:
             for group0 in arg0_groups:
-                if group0 not in arg._groups:
+                if group0 not in arg._groups_all:
                     if group0 == "observed_data":
                         continue
                     msg = "Mismatch between the groups."
                     raise TypeError(msg)
-            for group in arg._groups:
-                if group != "observed_data":
+            for group in arg._groups_all:
+                # handle data groups seperately
+                if group not in ["observed_data", "constant_data", "predictions_constant_data"]:
                     # assert that groups are equal
                     if group not in arg0_groups:
                         msg = "Mismatch between the groups."
@@ -500,7 +730,7 @@ def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
                     else:
                         inference_data_dict[group] = concatenated_group
                 else:
-                    # observed_data
+                    # observed_data, "constant_data", "predictions_constant_data",
                     if group not in arg0_groups:
                         setattr(arg0, group, deepcopy(group_data) if copy else group_data)
                         arg0._groups.append(group)
@@ -518,7 +748,7 @@ def concat(*args, dim=None, copy=True, inplace=False, reset_dim=True):
                     for var in group_vars:
                         if var not in group0_vars:
                             var_data = getattr(group_data, var)
-                            arg0.observed_data[var] = var_data
+                            getattr(arg0, group)[var] = var_data
                         else:
                             var_data = getattr(group_data, var)
                             var0_data = getattr(group0_data, var)
