@@ -2,15 +2,17 @@
 """General utilities."""
 import importlib
 import functools
+import re
 import warnings
-import numpy as np
-from numpy import newaxis
-import matplotlib.pyplot as plt
 
+import matplotlib.pyplot as plt
+import numpy as np
+
+from numpy import newaxis
 from .rcparams import rcParams
 
 
-def _var_names(var_names, data):
+def _var_names(var_names, data, filter_vars=None):
     """Handle var_names input across arviz.
 
     Parameters
@@ -18,15 +20,17 @@ def _var_names(var_names, data):
     var_names: str, list, or None
     data : xarray.Dataset
         Posterior data in an xarray
+    filter_vars: {None, "like", "regex"}, optional, default=None
+        If `None` (default), interpret var_names as the real variables names. If "like",
+         interpret var_names as substrings of the real variables names. If "regex",
+         interpret var_names as regular expressions on the real variables names. A la
+        `pandas.filter`.
+
     Returns
     -------
     var_name: list or None
     """
     if var_names is not None:
-
-        if isinstance(var_names, str):
-            var_names = [var_names]
-
         if isinstance(data, (list, tuple)):
             all_vars = []
             for dataset in data:
@@ -37,10 +41,7 @@ def _var_names(var_names, data):
         else:
             all_vars = list(data.data_vars)
 
-        excluded_vars = [i[1:] for i in var_names if i.startswith("~") and i not in all_vars]
-
-        all_vars_tilde = [i for i in all_vars if i.startswith("~")]
-
+        all_vars_tilde = [var for var in all_vars if var.startswith("~")]
         if all_vars_tilde:
             warnings.warn(
                 """ArviZ treats '~' as a negation character for variable selection.
@@ -50,18 +51,88 @@ def _var_names(var_names, data):
                 )
             )
 
-        if excluded_vars:
-            var_names = [i for i in all_vars if i not in excluded_vars]
+        try:
+            var_names = _subset_list(var_names, all_vars, filter_items=filter_vars, warn=False)
+        except KeyError as err:
+            msg = " ".join(("var names:", f"{err}", "in dataset"))
+            raise KeyError(msg)
+    return var_names
 
-        existent_vars = np.isin(var_names, all_vars)
-        if not np.all(existent_vars):
-            raise KeyError(
-                "{} var names are not present in dataset".format(
-                    np.array(var_names)[~existent_vars]
+
+def _subset_list(subset, whole_list, filter_items=None, warn=True):
+    """Handle list subsetting (var_names, groups...) across arviz.
+
+    Parameters
+    ----------
+    subset : str, list, or None
+    whole_list : list
+        List from which to select a subset according to subset elements and
+        filter_items value.
+    filter_items : {None, "like", "regex"}, optional
+        If `None` (default), interpret `subset` as the exact elements in `whole_list`
+        names. If "like", interpret `subset` as substrings of the elements in
+        `whole_list`. If "regex", interpret `subset` as regular expressions to match
+        elements in `whole_list`. A la `pandas.filter`.
+
+    Returns
+    -------
+    list or None
+        A subset of ``whole_list`` fulfilling the requests imposed by ``subset``
+        and ``filter_items``.
+    """
+    if subset is not None:
+
+        if isinstance(subset, str):
+            subset = [subset]
+
+        whole_list_tilde = [item for item in whole_list if item.startswith("~")]
+        if whole_list_tilde and warn:
+            warnings.warn(
+                "ArviZ treats '~' as a negation character for selection. There are "
+                "elements in `whole_list` starting with '~', {0}. Please double check"
+                "your results to ensure all elements are included".format(
+                    ", ".join(whole_list_tilde)
                 )
             )
 
-    return var_names
+        excluded_items = [
+            item[1:] for item in subset if item.startswith("~") and item not in whole_list
+        ]
+        filter_items = str(filter_items).lower()
+        not_found = []
+
+        if excluded_items:
+            if filter_items in ("like", "regex"):
+                for pattern in excluded_items[:]:
+                    excluded_items.remove(pattern)
+                    if filter_items == "like":
+                        real_items = [real_item for real_item in whole_list if pattern in real_item]
+                    else:
+                        # i.e filter_items == "regex"
+                        real_items = [
+                            real_item for real_item in whole_list if re.search(pattern, real_item)
+                        ]
+                    if not real_items:
+                        not_found.append(pattern)
+                    excluded_items.extend(real_items)
+            not_found.extend([item for item in excluded_items if item not in whole_list])
+            if not_found:
+                warnings.warn(
+                    f"Items starting with ~: {not_found} have not been found and will be ignored"
+                )
+            subset = [item for item in whole_list if item not in excluded_items]
+
+        else:
+            if filter_items == "like":
+                subset = [item for item in whole_list for name in subset if name in item]
+            elif filter_items == "regex":
+                subset = [item for item in whole_list for name in subset if re.search(name, item)]
+
+        existing_items = np.isin(subset, whole_list)
+        if not np.all(existing_items):
+            raise KeyError("{} are not present".format(np.array(subset)[~existing_items]))
+
+    return subset
 
 
 class lazy_property:  # pylint: disable=invalid-name
@@ -319,6 +390,37 @@ def expand_dims(x):
     return x.reshape(shape[:0] + (1,) + shape[0:])
 
 
+@conditional_jit(cache=True)
+def _dot(x, y):
+    return np.dot(x, y)
+
+
+def _cov_1d(x):
+    x = x - x.mean(axis=0)
+    ddof = x.shape[0] - 1
+    return np.dot(x.T, x.conj()) / ddof
+
+
+def _cov(data):
+    if data.ndim == 1:
+        return _cov_1d(data)
+    elif data.ndim == 2:
+        x = data.astype(float)
+        avg, _ = np.average(x, axis=1, weights=None, returned=True)
+        ddof = x.shape[1] - 1
+        if ddof <= 0:
+            warnings.warn("Degrees of freedom <= 0 for slice", RuntimeWarning, stacklevel=2)
+            ddof = 0.0
+        x -= avg[:, None]
+        prod = _dot(x, x.T.conj())
+        prod *= np.true_divide(1, ddof)
+        prod = prod.squeeze()
+        prod += 1e-6 * np.eye(prod.shape[0])
+        return prod
+    else:
+        raise ValueError("{} dimension arrays are not supported".format(data.ndim))
+
+
 @conditional_jit
 def full(shape, x, dtype=None):
     """Jitting numpy full."""
@@ -344,9 +446,12 @@ def flatten_inference_data_to_dict(
     var_names : str or list of str, optional
         Variables to be processed, if None all variables are processed.
     groups : str or list of str, optional
-        Select groups for CDS. Default groups are {"posterior_groups", "prior_groups"}
+        Select groups for CDS. Default groups are
+        {"posterior_groups", "prior_groups", "posterior_groups_warmup"}
             - posterior_groups: posterior, posterior_predictive, sample_stats
             - prior_groups: prior, prior_predictive, sample_stats_prior
+            - posterior_groups_warmup: warmup_posterior, warmup_posterior_predictive,
+                                       warmup_sample_stats
     ignore_groups : str or list of str, optional
         Ignore specific groups from CDS.
     dimension : str, or list of str, optional
@@ -393,8 +498,15 @@ def flatten_inference_data_to_dict(
             groups = ["posterior", "posterior_predictive", "sample_stats"]
         elif groups.lower() == "prior_groups":
             groups = ["prior", "prior_predictive", "sample_stats_prior"]
+        elif groups.lower() == "posterior_groups_warmup":
+            groups = ["warmup_posterior", "warmup_posterior_predictive", "warmup_sample_stats"]
         else:
-            raise TypeError("Valid predefined groups are {posterior_groups, prior_groups}")
+            raise TypeError(
+                (
+                    "Valid predefined groups are "
+                    "{posterior_groups, prior_groups, posterior_groups_warmup}"
+                )
+            )
 
     if dimensions is None:
         dimensions = "chain", "draw"
@@ -488,3 +600,67 @@ def flatten_inference_data_to_dict(
 
                         data_dict[var_name_dim] = var_values[loc]
     return data_dict
+
+
+def get_coords(data, coords):
+    """Subselects xarray DataSet or DataArray object to provided coords. Raises exception if fails.
+
+    Raises
+    ------
+    ValueError
+        If coords name are not available in data
+
+    KeyError
+        If coords dims are not available in data
+
+    Returns
+    -------
+    data: xarray
+        xarray.DataSet or xarray.DataArray object, same type as input
+    """
+    if not isinstance(data, (list, tuple)):
+        try:
+            return data.sel(**coords)
+
+        except ValueError:
+            invalid_coords = set(coords.keys()) - set(data.coords.keys())
+            raise ValueError("Coords {} are invalid coordinate keys".format(invalid_coords))
+
+        except KeyError as err:
+            raise KeyError(
+                (
+                    "Coords should follow mapping format {{coord_name:[dim1, dim2]}}. "
+                    "Check that coords structure is correct and"
+                    " dimensions are valid. {}"
+                ).format(err)
+            )
+    if not isinstance(coords, (list, tuple)):
+        coords = [coords] * len(data)
+    data_subset = []
+    for idx, (datum, coords_dict) in enumerate(zip(data, coords)):
+        try:
+            data_subset.append(get_coords(datum, coords_dict))
+        except ValueError as err:
+            raise ValueError("Error in data[{}]: {}".format(idx, err))
+        except KeyError as err:
+            raise KeyError("Error in data[{}]: {}".format(idx, err))
+    return data_subset
+
+
+def credible_interval_warning(credible_interval, hdi_prob):
+    """Replace credible_interval with hdi_prob and to warns of be deprecation."""
+    warnings.warn(
+        ("Keyword argument credible_interval has been deprecated " "Please replace with hdi_prob"),
+    )
+
+    if isinstance(credible_interval, str) and credible_interval == "auto":
+        raise Exception("Argument value 'auto' has been renamed to 'hide'")
+
+    if hdi_prob:
+        raise Exception(
+            "Both 'credible_interval' and 'hdi_prob' are in "
+            "keyword arguments. Please remove 'credible_interval'"
+        )
+
+    hdi_prob = credible_interval
+    return hdi_prob
